/*
	       File: WorkflowLoader.cs
	Create Date: 2007-09-06
	    Version:

	Description:


	 Change log:


*/
using System;
using System.IO;
using System.Xml;
using System.Reflection;
using System.Collections;

using System.Collections.Generic;
using System.Drawing.Design;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System.Windows.Forms;

using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.Activities.Rules;

namespace GuerillaProgrammer.FileProcessor
{
    public class WorkflowLoader : WorkflowDesignerLoader
    {
        #region Private member fields
        private String _markupFileName = String.Empty;
        private Type _newWorkflowType;
        private IToolboxService _toolboxService;
        private TypeProvider _typeProvider;
        private String _newWorkflowName = String.Empty;
        #endregion

        #region Constructors and cleanup methods
        public WorkflowLoader(TypeProvider typeProvider)
        {
            _typeProvider = typeProvider;
        }
        #endregion

        #region Private implementation methods
        protected override void Initialize()
        {
            base.Initialize();

            //add any necessary services
            IDesignerLoaderHost host = LoaderHost;
            if (host != null)
            {
                //add the custom MenuCommandService 
                host.AddService(typeof(IMenuCommandService),new WorkflowMenuService(host));

                //add the TypeProvider 
                host.AddService(typeof(ITypeProvider), _typeProvider, true);

                //add the toolbox service
                _toolboxService = new ToolBox(host);
                host.AddService(typeof(IToolboxService), _toolboxService);

                //add the property value UI service
                host.AddService(typeof(IPropertyValueUIService),new WorkflowPropertyValueService());

                //add the event binding service
                host.AddService(typeof(IEventBindingService),new WorkflowEventBindingService(host));
            }
        }
        protected override void PerformLoad(IDesignerSerializationManager serializationManager)
        {
            base.PerformLoad(serializationManager);
            Activity workflow = null;

            if (!String.IsNullOrEmpty(MarkupFileName))
            {
                //load a workflow from markup
                workflow = DeserializeFromMarkup(MarkupFileName);
            }
            else if (NewWorkflowType != null)
            {
                //create a new workflow
                workflow = CreateNewWorkflow(
                    NewWorkflowType, NewWorkflowName);
            }

            if (workflow != null)
            {
                IDesignerHost designer
                    = (IDesignerHost)GetService(typeof(IDesignerHost));
                //add the workfow definition to the designer
                AddWorkflowToDesigner(designer, workflow);
                //activate the designer
                designer.Activate();
            }
        }
        private Activity DeserializeFromMarkup(String fileName)
        {
            Activity workflow = null;

            //construct a serialization manager.
            DesignerSerializationManager dsm
                = new DesignerSerializationManager();
            using (dsm.CreateSession())
            {
                using (XmlReader xmlReader
                    = XmlReader.Create(fileName))
                {
                    //deserialize the workflow from the XmlReader
                    WorkflowMarkupSerializer markupSerializer
                        = new WorkflowMarkupSerializer();
                    workflow = markupSerializer.Deserialize(
                        dsm, xmlReader) as Activity;

                    if (dsm.Errors.Count > 0)
                    {
                        WorkflowMarkupSerializationException error
                            = dsm.Errors[0]
                              as WorkflowMarkupSerializationException;
                        throw error;
                    }
                }

                //deserialize a .rules file is one exists
                String rulesFileName = GetRulesFileName(fileName);
                if (File.Exists(rulesFileName))
                {
                    //read the .rules file
                    using (XmlReader xmlReader
                        = XmlReader.Create(rulesFileName))
                    {
                        WorkflowMarkupSerializer markupSerializer
                            = new WorkflowMarkupSerializer();
                        //deserialize the rule definitions
                        RuleDefinitions ruleDefinitions
                            = markupSerializer.Deserialize(dsm, xmlReader)
                                as RuleDefinitions;
                        if (ruleDefinitions != null)
                        {
                            //add the rules definitions to the workflow
                            workflow.SetValue(
                                RuleDefinitions.RuleDefinitionsProperty,
                                ruleDefinitions);
                        }
                    }
                }
            }

            return workflow;
        }
        private static String GetRulesFileName(String fileName)
        {
            String rulesFileName = Path.Combine(
                Path.GetDirectoryName(fileName),
                Path.GetFileNameWithoutExtension(fileName) + ".rules");
            return rulesFileName;
        }
        private Activity CreateNewWorkflow(Type workflowType,String newWorkflowName)
        {
            Activity workflow = null;

            ConstructorInfo cstr = workflowType.GetConstructor(Type.EmptyTypes);
            if (cstr != null)
            {
                workflow = cstr.Invoke(new Object[] { }) as Activity;
                workflow.Name = newWorkflowName;
                
                //set a default file name
                _markupFileName = newWorkflowName + ".xoml";
            }
            return workflow;
        }
        private static void AddWorkflowToDesigner(IDesignerHost designer, Activity workflow)
        {
            //add the root activity to the designer
            designer.Container.Add(workflow, workflow.QualifiedName);

            //add any child activities
            if (workflow is CompositeActivity)
            {
                List<Activity> children = new List<Activity>();
                //get a collection of all child activities
                GetChildActivities(
                    workflow as CompositeActivity, children);
                foreach (Activity child in children)
                {
                    designer.Container.Add(child, child.QualifiedName);
                }
            }
        }
        private static void GetChildActivities(CompositeActivity composite, List<Activity> children)
        {
            foreach (Activity activity in composite.Activities)
            {
                children.Add(activity);
                if (activity is CompositeActivity)
                {
                    //make recursive call
                    GetChildActivities(
                        activity as CompositeActivity, children);
                }
            }
        }
        protected override void PerformFlush(IDesignerSerializationManager serializationManager)
        {
            base.PerformFlush(serializationManager);

            //get the designer
            IDesignerHost designer
                = (IDesignerHost)GetService(typeof(IDesignerHost));

            //get the root activity of the workflow
            Activity workflow = designer.RootComponent as Activity;

            //serialize to a markup file
            if (workflow != null)
            {
                SerializeToMarkup(workflow, _markupFileName);
            }
        }
        private void SerializeToMarkup(Activity workflow, String fileName)
        {
            //clear the class name property since we are 
            //never creating a new class type.
            workflow.SetValue(
                WorkflowMarkupSerializer.XClassProperty, null);

            using (XmlWriter xmlWriter = XmlWriter.Create(fileName))
            {
                WorkflowMarkupSerializer markupSerializer
                    = new WorkflowMarkupSerializer();
                markupSerializer.Serialize(xmlWriter, workflow);
            }

            //Serialize rules if they exist
            RuleDefinitions ruleDefinitions = workflow.GetValue(
                RuleDefinitions.RuleDefinitionsProperty)
                    as RuleDefinitions;
            if (ruleDefinitions != null)
            {
                if (ruleDefinitions.Conditions.Count > 0 ||
                    ruleDefinitions.RuleSets.Count > 0)
                {
                    String rulesFileName = GetRulesFileName(fileName);
                    using (XmlWriter xmlWriter
                        = XmlWriter.Create(rulesFileName))
                    {
                        WorkflowMarkupSerializer markupSerializer
                            = new WorkflowMarkupSerializer();
                        markupSerializer.Serialize(
                            xmlWriter, ruleDefinitions);
                    }
                }
            }
        }
        #endregion

        #region Class properties
        public String MarkupFileName
        {
            get { return _markupFileName; }
            set { _markupFileName = value; }
        }
        public Type NewWorkflowType
        {
            get { return _newWorkflowType; }
            set { _newWorkflowType = value; }
        }
        public String NewWorkflowName
        {
            get { return _newWorkflowName; }
            set { _newWorkflowName = value; }
        }
        public TypeProvider TypeProvider
        {
            get { return _typeProvider; }
            set { _typeProvider = value; }
        }
        public override string FileName
        {
            get { return _markupFileName; }
        }
        #endregion

        #region Public interface
        public override System.IO.TextReader GetFileReader(string filePath)
        {
            return null;
        }
        public override System.IO.TextWriter GetFileWriter(string filePath)
        {
            return null;
        }
        public void RemoveFromDesigner(IDesignerHost designer, Activity workflow)
        {
            if (workflow != null)
            {
                designer.DestroyComponent(workflow);
                if (workflow is CompositeActivity)
                {
                    List<Activity> children = new List<Activity>();
                    //remove all child activities
                    GetChildActivities(
                        workflow as CompositeActivity, children);
                    foreach (Activity child in children)
                    {
                        designer.DestroyComponent(child);
                    }
                }
            }
        }
        public override void Flush()
        {
            PerformFlush(null);
        }
        #endregion
    }
}
